#
# Product:   Macal
# Author:    Marco Caspers
# Email:     SamaDevTeam@westcon.com
# License:   MIT License
# Date:      24-10-2023
#
# Copyright 2023 Westcon-Comstor
#

# This is the ast node for the foreach statement.

from __future__ import annotations
from .ast_nodetype import AstNodetype
from .lex_token import LexToken
from .ast_node_expression import AstNodeExpression
from .ast_node_statement import AstNodeStatement
from .ast_node_block import AstNodeBlock

class AstNodeForeachStatement(AstNodeStatement):
    def __init__(self, token: LexToken, expr: AstNodeExpression, block: AstNodeBlock) -> AstNodeForeachStatement:
        super().__init__(token)
        self.expr_type = AstNodetype.FOREACH_STATEMENT
        self.expr: AstNodeExpression = expr
        self.block: AstNodeBlock = block

    def __str__(self) -> str:
        return f"ForeachStatement({self.expr}, {self.block})"

    def __repr__(self) -> str:
        return self.__str__()

    def tree(self, indent: str = '') -> None:
        print(f"{indent}ForeachStatement:")
        self.expr.tree(indent + '    ')
        self.block.tree(indent + '    ')



#
# Product:   Macal
# Author:    Marco Caspers
# Email:     SamaDevTeam@westcon.com
# License:   MIT License
# Date:      24-10-2023
#
# Copyright 2023 Westcon-Comstor
#


from typing import List, Any
from .bytecode_register import BytecodeRegister

class BytecodeFlagsRegister(BytecodeRegister):
    def __init__(self, name: str, opcode: int) -> BytecodeRegister:
        super().__init__(name, opcode)
        self._carry: bool = False
        self._zero: bool = False
        self._sign: bool = False
        self._overflow: bool = False
        self._direction: bool = False
        self._interrupt: bool = False
        self._trap: bool = False
        self._supervisor: bool = False
        self._negative: bool = False
        self._index: bool = False

    def __str__(self) -> str:
        self._set_bit(6, self._zero)
        self._set_bit(0, self._carry)
        return f"CF {self._get_bit(0)} PF {self._get_bit(2)} ZF {self._get_bit(6)} SF {self._get_bit(7)} TF {self._get_bit(8)} DF {self._get_bit(10)} OF {self._get_bit(11)} IDX {self._get_bit(32)}"

    def _get_bit(self, bit: int) -> str:
        return '1' if (self.value & (1 << bit)) != 0 else '0'

    def __repr__(self) -> str:
        return self.__str__()

    def clear(self) -> None:
        self._carry = False # bit 0 (status)
        self._set_bit(1, True) # bit 1 is always true in EFLAGS
        self._parity = False # bit 2 (status)
        # bit 3 is reserved
        #self._auxiliary_carry = False # bit 4 (status)
        # bit 5 is reserved
        self._zero = False # bit 6 (status)
        self._sign = False # bit 7 (status)
        self._trap = False # bit 8 (single step) (control)
        #self._interrupt = False # bit 9 (interrupt enable) (control)
        self._direction = False # bit 10 (direction) (control)
        self._overflow = False # bit 11 (overflow) (status)
        self._index = False # bit 32 (status) (not a real flag, but used as such for the time being)
        # for RFLAGS bits 32-63 are reserved and must be 0

    def _set_bit(self, bit: int, value: bool) -> None:
        if value:
            self.value |= (1 << bit)
        else:
            self.value &= ~(1 << bit)

    @property
    def carry(self) -> bool:
        return self._carry

    @carry.setter
    def carry(self, value: bool) -> None:
        self._carry = value
        self._set_bit(0, value)

    @property
    def parity(self) -> bool:
        return self._parity

    @parity.setter
    def parity(self, value: bool) -> None:
        self._parity = value
        self._set_bit(2, value)

    @property
    def zero(self) -> bool:
        return self._zero

    @zero.setter
    def zero(self, value: bool) -> None:
        self._zero = value
        self._set_bit(6, value)

    @property
    def sign(self) -> bool:
        return self._sign

    @sign.setter
    def sign(self, value: bool) -> None:
        self._sign = value
        self._set_bit(7, value)

    @property
    def trap(self) -> bool:
        return self._trap

    @trap.setter
    def trap(self, value: bool) -> None:
        self._trap = value
        self._set_bit(8, value)

    @property
    def direction(self) -> bool:
        return self._direction

    @direction.setter
    def direction(self, value: bool) -> None:
        self._direction = value
        self._set_bit(10, value)

    @property
    def overflow(self) -> bool:
        return self._overflow

    @overflow.setter
    def overflow(self, value: bool) -> None:
        self._overflow = value
        self._set_bit(11, value)

    @property
    def index(self) -> bool:
        return self._index

    @index.setter
    def index(self, value: bool) -> None:
        self._index = value
        self._set_bit(32, value)

    def reset(self) -> None:
        self.clear()


#
# Product:   Macal
# Author:    Marco Caspers
# Email:     SamaDevTeam@westcon.com
# License:   MIT License
# Date:      24-10-2023
#
# Copyright 2023 Westcon-Comstor
#

# This is the elif statement ast node.

from __future__ import annotations
from typing import Optional
from .ast_node_expression import AstNodeExpression
from .ast_node_statement import AstNodeStatement
from .ast_nodetype import AstNodetype
from .ast_node_block import AstNodeBlock
from .lex_token import LexToken

class AstNodeElifStatement(AstNodeStatement):
    def __init__(self, token: LexToken) -> AstNodeElifStatement:
        super().__init__(token)
        self.expr_type = AstNodetype.ELIF_STATEMENT
        self.condition: Optional[AstNodeExpression] = None
        self.statements: AstNodeBlock = AstNodeBlock(token)

    def __str__(self) -> str:
        return f"AstNodeElifStatement({self.condition}, {self.statements})"

    def __repr__(self) -> str:
        return self.__str__()

    def tree(self, indent: str = '') -> None:
        print(f"{indent}ElifStatement:")
        self.condition.tree(indent + '    ')
        self.statements.tree(indent + '    ')


# Product:   Macal 2
# Author:    Marco Caspers
# Date:      16-10-2023
#
# String functions library

from typing import Any
from unidecode import unidecode

def StrLen(arg: str) -> int:
    """Implementation of len function"""
    return len(arg)

def StrLeft(arg: str, length: int) -> str:
    """Implementation of left function"""
    return arg[0:length]


def StrMid(arg: str, offset: int, length: int) ->str:
    """Implementation of mid function"""
    return arg[offset:(offset+length)]
    
def ToString(arg: Any) -> str:
    """Implementation of toString function"""
    return str(arg)        

def StrContains(needle: str, haystack: str) -> bool:
    """Implementation of strContains function"""
    return needle in haystack

def StrReplace(var: str, frm: str, wth: str) -> str:
    """Implementation of strReplace function"""
    return var.replace(frm, wth)


def StartsWith(haystack: str, needle: str) -> bool:
    """Implementation of StartsWith function"""
    return haystack.startswith(needle)

def RemoveNonAscii(text: str) -> str:
    """Implementation of RemoveNonAscii function"""
    result = unidecode(text)

def ReplaceEx(var: str, repl: Any, by: str) -> str:
    """Implementation of ReplaceEx function"""
    result = var
    for ch in repl:
        result = result.replace(ch, by)
    return result

def PadLeft(string: str, char: str, amount: int) -> Any:
    """Implementation of PadLeft function"""
    # this is counter intuitive, but the *just functions in python pad the character on the other 
    # end as what their name would imply.
    return string.rjust(amount, char)

def PadRight(string: str, char: str, amount: int) -> Any:
    """Implementation of PadRight function"""
    # this is counter intuitive, but the *just functions in python pad the character on the other 
    # end as what their name would imply.
    return string.ljust(amount, char)
    
def PadCenter(string: str, char: str, amount: int) -> Any:
    """Implementation of PadCenter function"""
    return string.center(amount, char)